@@ -9,13 +9,29 @@ from typing import List, Optional
 from ._metadata import APP
 
 
+class GitException(Exception):
+    pass
+
+
+class CheckoutFailed(GitException):
+    pass
+
+
+class PullFailed(GitException):
+    pass
+
+
 class Git:
     _GIT = 'git'
 
+    _APP_EMAIL = f'{APP}@localhost'
+    _ARBITRARY_FIXED_DATETIME = '2005-12-21T00:00:00+00:00'  # release date of Git 1.0.0
     _COMMIT_ENVIRON = {
-        'GIT_AUTHOR_EMAIL': f'{APP}@localhost',
+        'GIT_AUTHOR_DATE': _ARBITRARY_FIXED_DATETIME,
+        'GIT_AUTHOR_EMAIL': _APP_EMAIL,
         'GIT_AUTHOR_NAME': APP,
-        'GIT_COMMITTER_EMAIL': f'{APP}@localhost',
+        'GIT_COMMITTER_DATE': _ARBITRARY_FIXED_DATETIME,
+        'GIT_COMMITTER_EMAIL': _APP_EMAIL,
         'GIT_COMMITTER_NAME': APP,
     }
 
@@ -161,16 +177,24 @@ class Git:
         argv = [self._GIT, 'remote', 'update', '--prune', remote_name]
         self._subprocess_check_output(argv, is_write=True)
 
-    def checkout(self, committish: str, new_branch: Optional[str] = None) -> None:
+    def checkout(self, committish: str) -> None:
         argv = [self._GIT, 'checkout', '-q']
-        if new_branch is not None:
-            argv += ['-b', new_branch]
         argv.append(committish)
-        self._subprocess_check_output(argv, is_write=True)
+        try:
+            self._subprocess_check_output(argv, is_write=True)
+        except subprocess.CalledProcessError as e:
+            if e.returncode == 1:
+                raise CheckoutFailed
+            raise
 
     def pull_ff_only(self) -> None:
         argv = [self._GIT, 'pull', '--ff-only']
-        self._subprocess_check_output(argv, is_write=True)
+        try:
+            self._subprocess_check_output(argv, is_write=True)
+        except subprocess.CalledProcessError as e:
+            if e.returncode == 1:
+                raise PullFailed
+            raise
 
     def _has_changes(self, extra_argv: Optional[List[str]] = None) -> bool:
         argv = [self._GIT, 'diff', '--exit-code', '--quiet']
@@ -202,11 +226,18 @@ class Git:
         cherry_lines = self.cherry(with_regard_to, branch)
         return any(line.startswith('+') for line in cherry_lines)
 
-    def commit(self, message: str) -> None:
-        argv = [self._GIT, 'commit', '--no-verify', '-m', message]
+    def commit_tree(self, message: str, parent_committish: str, tree: str) -> str:
+        argv = [self._GIT, 'commit-tree', '-m', message, '-p', parent_committish, tree]
         env = os.environ.copy()
         env.update(self._COMMIT_ENVIRON)
-        self._subprocess_check_output(argv, env=env, is_write=True)
+        # Note: Command "git commit-tree" does write to the repository but it does
+        #       not switch branches, move HEAD or delete things; that's why it
+        #       it is considered "not writing" (``is_write=False``) here and
+        #       will be performed even when ``--dry-run``/``self._pretend`` is active.
+        output_bytes = self._subprocess_check_output(argv, env=env, is_write=False)
+        lines = self._output_bytes_to_lines(output_bytes)
+        assert len(lines) == 1
+        return lines[0]
 
     def merge_base(self, target_branch, topic_branch) -> str:
         argv = [self._GIT, 'merge-base', target_branch, topic_branch]
@@ -214,8 +245,3 @@ class Git:
         lines = self._output_bytes_to_lines(output_bytes)
         assert len(lines) == 1
         return lines[0]
-
-    def squash_merge(self, committish: str) -> None:
-        # NOTE: "git merge --squash <committish>" does not create a commit
-        argv = [self._GIT, 'merge', '--squash', committish]
-        self._subprocess_check_output(argv, is_write=True)
